#ifndef __NQS_QUEUE_DEV_HEADER
#define __NQS_QUEUE_DEV_HEADER

/*
 * nqs_queue/all-systems/DEV.h
 * Device status flags for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	Device status flags.
 */
#define	DEV_ACTIVE	0000001		/* Device active and handling request,*/
					/* Otherwise device is inactive, or */
					/* failed */
#define	DEV_ENABLED	0000002		/* Device is enabled, otherwise */
					/* device is disabled. */
#define	DEV_FAILED	0000004		/* If device fails, then this bit */
					/* is set and the DEV_ACTIVE and */
					/* DEV_ENABLED bits are cleared */

#endif /* __NQS_QUEUE_DEV_HEADER */
