#ifndef __NQS_QUEUE_SEL_HEADER
#define __NQS_QUEUE_SEL_HEADER

/*
 * nqs_queue/all-systems/SEL.h
 * Request selection types for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	Request selection type:
 */
#define	SEL_REQNAME	0000001		/* Selection entry is by request-name */
#define	SEL_REQID	0000002		/* Selection entry is by request-id */
#define	SEL_FOUND	0100000		/* Selection entry has been found */

#endif /* __NQS_QUEUE_SEL_HEADER */
