#ifndef __NQS_QUEUE_SHELL_HEADER
#define __NQS_QUEUE_SHELL_HEADER

/*
 *	Batch request shell script choice modes.
 *	(See nqs_spawn.c and shserver.c)
 */
#define	SHSTRAT_FREE	0		/* Let the user's login shell choose */
					/* which shell (e.g. csh, sh, ksh), */
					/* should be used to run the batch */
					/* request script */
#define	SHSTRAT_FIXED	1		/* Run batch request scripts with */
					/* site designated batch request */
					/* shell */
#define	SHSTRAT_LOGIN	2		/* Run batch request scripts with */
					/* the user's login shell */

#endif /* __NQS_QUEUE_SHELL_HEADER */
