#ifndef __NQS_DB_ALLODB_HEADER
#define __NQS_DB_ALLODB_HEADER

/*
 * nqs_db/all-systems/allodb.h
 * Header file for allodb.c
 */

#include <nqs_generic/license.h>

#include <errno.h>

#endif /* __NQS_DB_ALLODB_HEADER */
