#ifndef __NQS_GENERIC_MKREQ_HEADER
#define __NQS_GENERIC_MKREQ_HEADER

/*
 * nqs_generic/all-systems/mkreq.h
 * Completion codes for making requests
 */

#include <nqs_generic/license.h>

#define	MKREQ_SUCCESS	 0
  /* Control file successfully created */
#define	MKREQ_NOCREATE	 (-1)
  /* Unable to open/create control file */
#define	MKREQ_NOMID	 (-2)
  /* Unable to get machine-id of local host */
#define	MKREQ_BADREQTYPE (-3)
  /* Bad request type specified */
#define	MKREQ_NOUSERNAME (-4)
  /* Could not get username for user-id */
#define	MKREQ_NOCWD	 (-5)
  /* Unable to get current working directory */
#define	MKREQ_CWDNEWLINE (-6)
  /* Current working directory contained a new line character */
#define	MKREQ_NOCHDIRNEW (-7)
  /* Unable to chdir() to the NQS new request directory */
#define	MKREQ_NOLOCALDAE (-8)
  /* Unable to get pipe to local daemon */
#define	MKREQ_NOSETUGID	 (-9)
  /* Unable to set [uid,gid] */
#define	MKREQ_NOPARMFILE (-10)
  /* Unable to open NQS parameters file */
#define MKREQ_INVALIDACC (-11)
  /* Invalid account ID */

#endif /* __NQS_GENERIC_MKREQ_HEADER */
