/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Mon Nov  9 15:41:20 1992
 */
#ifndef __CEXTRACT__
#if __STDC__

int machine_name ( Mid_t mid, char *path );
int namop ( char *name, Mid_t *mid, int mode );
int nmap_add_alias ( char *alias, char *name );
int nmap_add_gid ( Mid_t from_mid, gid_t from_gid, gid_t local_gid );
int nmap_add_host ( char *name );
int nmap_add_mid ( Mid_t mid, char *name );
int nmap_add_nam ( char *from_nam, Mid_t to_mid );
int nmap_add_uid ( Mid_t from_mid, uid_t from_uid, uid_t local_uid );
int nmap_chg_mid ( Mid_t mid, char *name );
int nmap_create ( void );
void nmap_ctl ( int opcode, int parameter );
int nmap_del_defgid ( Mid_t from_mid );
int nmap_del_defuid ( Mid_t from_mid );
int nmap_del_gid ( Mid_t from_mid, gid_t from_gid );
int nmap_del_host ( char *name );
int nmap_del_mid ( Mid_t mid );
int nmap_del_nam ( char *from_nam );
int nmap_del_uid ( Mid_t from_mid, uid_t from_uid );
int nmap_dis_mid ( Mid_t from_mid, time_t wakeuptime );
int nmap_ena_mid ( Mid_t from_mid );
int nmap_get_defuid ( Mid_t from_mid, uid_t *defuid );
struct hostent *nmap_get_ent ( Mid_t mid );
int nmap_get_gid ( Mid_t from_mid, gid_t from_gid, gid_t *local_gid );
int nmap_get_mid ( struct hostent *hostent, Mid_t *mid );
char *nmap_get_nam ( Mid_t mid );
int nmap_get_uid ( Mid_t from_mid, uid_t from_uid, uid_t *local_uid );
int nmap_jrr_mid ( Mid_t mid );
int nmap_list_nam ( void );
u_long nmap_ntoml ( char *host );
int nmap_set_defgid ( Mid_t from_mid, gid_t defgid );
int nmap_set_defuid ( Mid_t from_mid, uid_t defuid );
int nmap_show ( void );
int open_machine ( Mid_t mid, int access );
int open_name_map ( int access );
int xidop ( Mid_t mid, struct xidparm *parm, int mode );

#else /* __STDC__ */

int machine_name (/* Mid_t mid, char *path */);
int namop (/* char *name, Mid_t *mid, int mode */);
int nmap_add_alias (/* char *alias, char *name */);
int nmap_add_gid (/* Mid_t from_mid, gid_t from_gid, gid_t local_gid */);
int nmap_add_host (/* char *name */);
int nmap_add_mid (/* Mid_t mid, char *name */);
int nmap_add_nam (/* char *from_nam, Mid_t to_mid */);
int nmap_add_uid (/* Mid_t from_mid, uid_t from_uid, uid_t local_uid */);
int nmap_chg_mid (/* Mid_t mid, char *name */);
int nmap_create (/* void */);
void nmap_ctl (/* int opcode, int parameter */);
int nmap_del_defgid (/* Mid_t from_mid */);
int nmap_del_defuid (/* Mid_t from_mid */);
int nmap_del_gid (/* Mid_t from_mid, gid_t from_gid */);
int nmap_del_host (/* char *name */);
int nmap_del_mid (/* Mid_t mid */);
int nmap_del_nam (/* char *from_nam */);
int nmap_del_uid (/* Mid_t from_mid, uid_t from_uid */);
int nmap_dis_mid (/* Mid_t from_mid, time_t wakeuptime */);
int nmap_ena_mid (/* Mid_t from_mid */);
int nmap_get_defuid (/* Mid_t from_mid, uid_t *defuid */);
struct hostent *nmap_get_ent (/* Mid_t mid */);
int nmap_get_gid (/* Mid_t from_mid, gid_t from_gid, gid_t *local_gid */);
int nmap_get_mid (/* struct hostent *hostent, Mid_t *mid */);
char *nmap_get_nam (/* Mid_t mid */);
int nmap_get_uid (/* Mid_t from_mid, uid_t from_uid, uid_t *local_uid */);
int nmap_jrr_mid (/* Mid_t mid */);
int nmap_list_nam (/* void */);
u_long nmap_ntoml (/* char *host */);
int nmap_set_defgid (/* Mid_t from_mid, gid_t defgid */);
int nmap_set_defuid (/* Mid_t from_mid, uid_t defuid */);
int nmap_show (/* void */);
int open_machine (/* Mid_t mid, int access */);
int open_name_map (/* int access */);
int xidop (/* Mid_t mid, struct xidparm *parm, int mode */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
