#ifndef __NQS_QUEUE_RTS_HEADER
#define __NQS_QUEUE_RTS_HEADER

/*
 * nqs_queue/all-systems/RTS.h
 * Request transaction states for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	NQS request transaction states (RTS_).
 *	WARNING:  These values must be in [0..7].
 */
#define	RTS_STASIS	0		/* This request is not doing any- */
					/* thing interesting at the moment */
#define	RTS_PREARRIVE	1		/* Preparing for arrive state */
#define	RTS_ARRIVE	2		/* Arriving state */
#define	RTS_PREDEPART	3		/* Preparing for depart state */
#define	RTS_DEPART	4		/* Departing state */
#define	RTS_STAGEIN	5		/* Staging-in state */
#define	RTS_EXECUTING	6		/* Executing state */
#define	RTS_STAGEOUT	7		/* Staging-out state */

#endif /* __NQS_QUEUE_RTS_HEADER */
