/*
 * nqs_generic/all-systems/license.h
 * License information for the Monsanto-NQS source code.
 * 
 * This file is included by all other source code files in
 * Monsanto-NQS.  It should only be modified by the current 
 * maintainer.
 */

/*
 * Monsanto-NQS
 * The Network Queueing System (NQS)
 * 
 * Copyright (c) 1985 NASA
 * Copyright (c) 1992 John Roman, Monsanto Company
 * Copyright (c) 1994, 1995 The University of Sheffield
 * 
 * Portions are the copyright of individual contributors.
 * 
 * Work done by the University of Sheffield is funded by the Joint
 * Information Services Committee New Technologies Initiative grant
 * NTI/48.2, on behalf of all UK Higher Education sites.  More
 * information is available from :
 * 
 * 	http://www.shef.ac.uk/uni/projects/nqs/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This license is to be found in the file COPYING, which is in the
 * top directory of the source code distribution.
 */

static char *szCopyrightString = "Copyright (c) 1985 NASA.\nCopyright (c) 1992 John Roman, Monsanto Company.\nCopyright (c) 1994, 1995 The University of Sheffield.\nAll rights reserved."
