#ifndef __NQS_QUEUE_PIPE_HEADER
#define __NQS_QUEUE_PIPE_HEADER

/*
 * nqs_queue/all-systems/PIPE.h
 * Pipe queue destination status flags
 */

#include <nqs_generic/license.h>

/*
 *	Pipe queue destination status flags.
 */
#define	DEST_ENABLED	0000002		/* Pipe queue dest enabled */
#define	DEST_FAILED	0000004		/* Pipe queue dest failed */
#define	DEST_RETRY	0000010		/* Pipe queue dest in retry mode */

#endif /* __NQS_QUEUE_PIPE_HEADER */
