#ifndef __NQS_QUEUE_REQ_HEADER
#define __NQS_QUEUE_REQ_HEADER

/*
 * nqs_queue/all-systems/REQ.h
 * Request file magic numbers for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	NQS request header magic number(s).
 */
#define	REQ_MAGIC1	1618033989	/* Magic number used to verify */
					/* valid request control file (at */
					/* least to verify the first block */
					/* as valid) */
#endif /* __NQS_QUEUE_REQ_HEADER */
